/*
 * This file is part of the R&S 1MA226 application note.
 *
 * Copyright (C) 2016 Rohde & Schwarz <www.rohde-schwarz.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */

// software tool for INSTR status reload
// according to application note 1MA_226
// Author: Dieter Bues

// Project: c:\Dev-Cpp\Examples\1MA226_LoadStat\LoadStat.dev
// development system used:  Dev-Cpp 5.11 TDM-GCC 4.9.2 Setup.exe

#include <cstdlib>
#include <iostream>
#include <string>

#include <time.h>

#include "visa.h"
#define LargeBuffLen 200000
#define SmallBuffLen 256
#define DEBUG_ON 128

#pragma warning(disable : 4996)

#define version "Instrument status reload  v. 1.1, R&S 1MAA_DB, July 12th, 2016"

// Revision history:
// listed from top to bottom, i.e. the actual one is at the end of the list
 
// First release:  Oct 31th, 2013   
            
// (1) FSV implementation
//     On request from Alan Tay / RSSG the instrument FSV will be
//     implemented in addition to ZVx and FSW.
//     Appropriate version message of LoadStat:  1.1
//     Test parameter : (to be set in DEV-CPP / Execute / Parameter)
//     db1 128 TCPIP0::fsv7-102622::inst0::INSTR
//  Tag: [db_000]
//  Date:  June 8th, 2016

using namespace std;

 ViSession instr;
 ViSession defaultRM;
 ViStatus status;
 ViUInt32 retCount;
 ViUInt32 writeCount;
 unsigned char buffer[SmallBuffLen];
 char stringinput[SmallBuffLen];
 

 bool debugx = false;
//const bool debugx = false;


// implemented instruments :
enum INSTR_TYPE
{
    ZVX,
    FSW,
    FSV,   // [db_000]
    UNK
};

INSTR_TYPE act_instr;



enum {
  status_OK,
  status_INVALID_NUMBER_OF_PARAMETERS,
  status_REM_IO_ERR,
  status_REM_OPEN_ERR,
  status_INVALID_ADDRESS,
  status_REM_DAT_READ_ERR,
  status_INVALID_SAVESET,
  status_REM_DAT_FORMAT_ERR,
  status_UNKNOWN_INSTRUMENT
};

//string par_1("TCPIP::10.110.10.253::INSTR");
string par_1("TCPIP::LOCALHOST::INSTR");
//string par_1("TCPIP::10.110.10.164::INSTR");
string par_2("aaabbbcccdddeee");
//string par_3 =("1");

unsigned int status_code = status_OK;   // suppose everything is OK


// standard remote open function 
ViSession RemOpen (string remote_address)
{
         ViSession instr;
         //ViSession defaultRM;
         ViStatus status;
         char addr_buff[100];
         
         int x,i;

        x=remote_address.find("INSTR");
        
        if (x>0)
        {
            for(i=0;i< x+5;i++)
                addr_buff[i]= remote_address[i];
            addr_buff[i]=0;
                
                status=viOpenDefaultRM (&defaultRM);
                if (status < VI_SUCCESS)
                  status_code = status_REM_OPEN_ERR;
                 
                //status = viOpen (defaultRM, (ViRsrc)  "TCPIP::10.110.10.44::INSTR", VI_NULL, VI_NULL, &instr);
                //
                status = viOpen (defaultRM, (ViRsrc) addr_buff, VI_NULL, VI_NULL, &instr);
                status = viSetAttribute (instr, VI_ATTR_TMO_VALUE, 500);
                return(instr);
        }
        status_code = status_INVALID_ADDRESS;
		return instr;
}

// standard remote output function 
void RemOut (ViSession instr,string inpar)
{
    ViUInt32 writeCount;
    ViStatus status;
    char stringinput[512];
    unsigned int i;
    unsigned int x;
    
    for(i=0;i<inpar.length();i++)
        stringinput[i]=inpar[i];
    
    stringinput[i]=0;

    x=inpar.length();

    status = viWrite (instr, (ViBuf)stringinput, (ViUInt32) x, &writeCount);
    if (status < VI_SUCCESS)
       status_code = status_REM_IO_ERR;
}



// standard remote input function
// for standard SCPI communication only
string RemInp (ViSession instr)
{
   ViUInt32 retCount;
   ViStatus status;
   unsigned char buffer[SmallBuffLen];
   string xx="                                                                                    ";
   int i;
   
   status = viRead (instr, buffer, sizeof buffer, &retCount);

   if (status < VI_SUCCESS)
    {
       status_code = status_REM_IO_ERR;
    }

    //xx.clear();
    for(i=0;i<(int)retCount;i++)
        xx[i]=buffer[i];
    xx[i]=0;

    return(xx);
}

// standard remote close function
void RemClose (ViSession instr)
{
   ViStatus status;         
   
   status = viClose(instr);
   status = viClose(defaultRM);
}


// replacement for stoi which is available
// in newer compiler versions
int xstoi (string inpar)
{
  int i=0;
  int x;
  char buff[10];
  
  while((inpar[i]!=0) && (i<10))
  {
    buff[i]=inpar[i++];
  }
  buff[++i]=0;   // set string delimiter
  x=atoi(buff);
  return(x);
}




// ************************ main program **************************
int main(int argc, char *argv[])
{
    int x;
	unsigned int i;
    string result;
    string xx ="                                                                             ";
    string yy;
    int stat_STB, stat_ESR;
    long byte_count,n;
    char dat_buff[LargeBuffLen];
    ViSession INSTR;

    if (argc==1)   // do nothing if no parameters are available
       goto xend;

	if (argc > 2)
	{
		int debugInt = atoi(argv[2]);
		if((debugInt & DEBUG_ON) == DEBUG_ON)
			debugx = true;

        // documentation for 3. parameter :
        // Using this  parameter a remote address
        // can be specified in order to operate this software
        // from an external PC.  This is useful for debugging sessions
        // When the 3. parameter is specified, it will be used instead of 
        // the local host address.
        // If tbe 3. parameter is specified, the second parameter has to
        // be specified also, normally = 128
        // example for a valid call using the 3. parameter :
        // LoadStat_NWA  reteid_00 128 TCPIP::10.110.10.164::INSTR
            			
		if (argc == 4)   // remote address specifier (undocumented parameter !)
		  {
            par_1 = argv[3];  // overwrite local host address
          }
	}

    if (debugx)
     {
       cout << version  << endl;
       cout << endl;
       }
    
    if (debugx)
      {
       cout << "**1** number of command line parameters:  ";         
       cout << argc -1 << endl;    // decreased by 1 because program name doesn't count
      }

     if (debugx)
     {
      cout << endl << "**2** commands detected :" << endl;
      for(i=1;i<argc;i++)
        {
          cout << i << ": ";                 
          cout << argv[i] << endl;
         }
      cout <<  endl;
     }

         INSTR=RemOpen(par_1);
         
   
         RemOut(INSTR,"*IDN?");
             
  
         xx=RemInp(INSTR);
         if (debugx)
           {
            cout << endl;
            cout << "**3** remote instrument detected :" << endl;
            cout << xx << endl;
            cout << endl;           
           }

          if ((long)xx.find("FSW")>=1)
            {
             act_instr=FSW;
             //cout << (long)xx.find("FSW") << endl;
             //cout << xx.length() << endl;
            }
          else 
           if ((long)xx.find("ZV")>=1)
            {
            act_instr=ZVX;
             //cout << (long)xx.find("ZV") << endl;
             //cout << xx.length() << endl;
            }
          else 
           if ((long)xx.find("FSV")>=1)  // [db_000]
            {
            act_instr=FSV;
            }
          else
            act_instr=UNK;      // unknown instrument


         par_1 = (string) argv[1];
         
         switch (act_instr)
          {
            case ZVX:
                 RemOut(INSTR,"SYSTem:ERRor:ALL?");  // clear error queue
                 xx=RemInp(INSTR);
                 par_2 = "MMEM:LOAD:STAT 1,'C:\\Rohde&Schwarz\\NWA\\RecallSets\\"+par_1+".zvx\'";
                 break;
                 
            case FSW:
                 RemOut(INSTR,"SYSTem:ERRor:ALL?");  // clear error queue
                 xx=RemInp(INSTR);
                 
                 par_2 = "MMEM:LOAD:STAT 1,'C:\\R_S\\Instr\\user\\"+par_1+".dfl\'";
                 break;
            
            // [db_000]     
            case FSV:
                 
                 // clear error queue
                 // in contrast to other instruments FSV
                 // requires an explicite reading of the 
                 // error queue until result is zero
                 do 
                 {
                   RemOut(INSTR,"SYSTem:ERRor?");  
                   xx=RemInp(INSTR);
                 }
                 while(xstoi(xx) != 0);
                 // does  n o t   really clean the  eq :
                 //RemOut(INSTR,"SYSTem:ERRor:ALL?");  // clear error queue
                 //xx=RemInp(INSTR);
                 
                 RemOut(INSTR,"MMEM:SEL:ALL");  // save and restore all items
                 //xx=RemInp(INSTR);

                 par_2 = "MMEM:LOAD:STAT 1,'C:\\R_S\\Instr\\user\\"+par_1+"\'";
                 break;
                 
            case UNK:
                 status_code =status_UNKNOWN_INSTRUMENT;
                 goto err_exit;
                 break;
          }

         
       if (debugx)
         {
           cout << "**4** status load command to INSTR :" << endl;
           cout << par_2 << endl;
           cout << endl;
          }

		 RemOut(INSTR,par_2);               // load instrument status

         RemOut(INSTR,"*STB?");
         xx = RemInp(INSTR);
         

         
         //stat_STB = (int)xx[0] -0x30;    // conversion of 8-bit statusbyte
         stat_STB = xstoi(xx);


		 
         RemOut(INSTR,"*ESR?");
         yy = RemInp(INSTR);
         

         stat_ESR = xstoi(yy);    // conversion of 8-bit error status

		 
		 // remote interface status evaluation :
         switch (act_instr)
          {
            case ZVX:
        		 if ((stat_STB == 0) && (stat_ESR == 0))
        		   {
        		   status_code = status_OK;
        		   RemOut(INSTR,"DISP:MENU:KEY:SEL 'OSP_Panel.bat'");  // reactivate the External Tools softkey column
                   }
                 else
                   {
                     if (((stat_STB & 4)==4) && ((stat_ESR & 16) ==16))
                       status_code = status_INVALID_SAVESET;
                   }
               break;
                 
            case FSW:
        		 if (((stat_STB == 0) && (stat_ESR == 0)) || ((stat_STB == 4) && (stat_ESR == 32)) )
        		   {
        		   status_code = status_OK;
                   }
                 else
                   {
                     if (((stat_STB & 4)==4) && ((stat_ESR & 16) ==16))
                       status_code = status_INVALID_SAVESET;
                   }
               break;

               // [db_000]
            case FSV:
        		 //if (((stat_STB == 0) && (stat_ESR == 0)) || ((stat_ESR & 36) == 36))
        		 if (stat_STB == 0)
        		   {
        		   status_code = status_OK;
                   }
                 else
                   {
                     if (((stat_STB & 4)==4) && ((stat_ESR & 16) ==16))   // tested in June 2016: ok
                       status_code = status_INVALID_SAVESET;
                   }
               break;
          }
		 
		 
         if (debugx)
         {
          cout << "**5** INSTR status byte & error status register :" << endl;
          cout << "Status STB :" ;
          cout << stat_STB << endl;
    
          cout << "Status ESR :" ;
          cout << stat_ESR << endl << endl;
         }	 

         RemClose(INSTR);
 
        
// status evaluation :
err_exit:
         
if (debugx)
 if(status_code == status_OK)
   {
     cout << "**6** final status :" << endl;
     cout << "INSTR status reload successfull !"  << endl;
   }
 else
   {
     switch (status_code)
     {
        case status_INVALID_ADDRESS:
             cout << "Invalid remote control address,  must include the term 'INSTR'" << endl;
             break;
             
        case   status_REM_IO_ERR:
             cout << "It was not possible to access the remote control interface" << endl;
             break;

             
        case   status_INVALID_SAVESET:
             cout << "The specified setup file was not available in INSTR" << endl;
             break;
      
      
        case   status_INVALID_NUMBER_OF_PARAMETERS:
             cout << "The number of parameters in the command line must be either 1,2 or 3" << endl;
             cout << "Please check the command line for ambiguous space characters" << endl;
             break;


        case   status_REM_OPEN_ERR:
             cout << "An error occured during opening the VISA  io-channel" << endl;
             break;

         case status_UNKNOWN_INSTRUMENT:
             cout << "The instrument has not yet been implemented for setup reload" << endl;
             break;
              
              
        default:
             cout << "An unknown error occured" << endl;
            
        }  // end switch
     }  // end else    
  if (debugx)
   {
    cout << endl; 
    system("PAUSE");
   }   
xend:    
  return EXIT_SUCCESS;    
}




